const highp float zNear = 0.1;
const highp float zFar = 1000.0;

highp float linearDepth(highp float depthSample)
{
	return 2.0 * zNear * zFar / (zFar + zNear - (2.0 * depthSample - 1.0) * (zFar - zNear));
}

highp float depthSample(highp float linearDepth)
{
	return ((zFar + zNear - 2.0 * zNear * zFar / linearDepth) / (zFar - zNear) + 1.0) / 2.0;
}					

highp float swCube_getDepth(highp sampler2D depthMap, highp vec3 dir, out highp vec3 cubePoint)
{
	highp vec3 av = abs(dir);
	
	highp vec3 faceAbsNor = mix(vec3(1.0, 0.0, 0.0), vec3(0.0, 1.0, 0.0), step(av.x, av.y));
	faceAbsNor = mix(faceAbsNor, vec3(0.0, 0.0, 1.0), step(dot(av, faceAbsNor), av.z));

	highp vec3 faceNor = sign(dir) * faceAbsNor;
	cubePoint = dir / dot(faceAbsNor, av);

	highp vec4 x0_y0_x1_y1;
	x0_y0_x1_y1.x = dot(cubePoint, faceNor.yzx);
	x0_y0_x1_y1.y = dot(cubePoint, faceAbsNor.zxy);
	x0_y0_x1_y1.z = dot(faceAbsNor, vec3(0.0, 0.25, 0.5));
	x0_y0_x1_y1.w = dot(faceNor, vec3(1.0));

	x0_y0_x1_y1 *= vec4(0.125, 0.25, 1.0, -0.25);
	x0_y0_x1_y1 += vec4(0.125, 0.25, 0.0, 0.25);

	return decodeFloat16(texture2D(depthMap, x0_y0_x1_y1.xy + x0_y0_x1_y1.zw).rg);
}

highp float swCube_testPoint(highp sampler2D depthMap, highp vec3 p)
{
	highp vec3 cubePoint;
	highp float dist = linearDepth(swCube_getDepth(depthMap, p, cubePoint));
	highp vec3 depthPoint = cubePoint * dist;
	return step(depthPoint.z, p.z + 0.1);
	
	//highp float pointDist = dot(faceNor, p);
	//return step(pointDist, cubeDist + 0.1);
}

highp float testPoint(highp sampler2D depthMap, highp vec3 pos, highp vec2 offset)
{
	pos.xy += offset;
	return swCube_testPoint(depthMap, pos);
}

highp float testPoint_4(highp sampler2D depthMap, highp vec3 pos, highp float radius, highp vec2 lmPos)
{
	highp float radians = random(lmPos) * 3.1415;
	highp float c = cos(radians);
	highp float s = sin(radians);

	highp mat2 rot;
	rot[0] = vec2(c, -s) * radius;
	rot[1] = vec2(s, c) * radius;

	highp float vis = 0.0;
	vis += testPoint(depthMap, pos, rot * vec2(  0.0, -1.0 ));
	vis += testPoint(depthMap, pos, rot * vec2( -1.0,  0.0 ));
	vis += testPoint(depthMap, pos, rot * vec2(  1.0,  0.0 ));
	vis += testPoint(depthMap, pos, rot * vec2(  0.0,  1.0 ));

	return vis / 4.0;
}

highp float testPoint_4_no_rot(highp sampler2D depthMap, highp vec3 pos, highp float radius, highp vec2 lmPos)
{
	highp float vis = 0.0;
	vis += testPoint(depthMap, pos, vec2(  0.0, -1.0 ) * radius);
	vis += testPoint(depthMap, pos, vec2( -1.0,  0.0 ) * radius);
	vis += testPoint(depthMap, pos, vec2(  1.0,  0.0 ) * radius);
	vis += testPoint(depthMap, pos, vec2(  0.0,  1.0 ) * radius);

	return vis / 4.0;
}

highp float testPoint_5(highp sampler2D depthMap, highp vec3 pos, highp mat2 rot)
{
	highp float vis = 0.0;
	vis += testPoint(depthMap, pos, vec2(0.0, 0.0));
	vis += 0.75 * testPoint(depthMap, pos, rot * vec2(  0.0, -1.0 ));
	vis += 0.75 * testPoint(depthMap, pos, rot * vec2( -1.0,  0.0 ));
	vis += 0.75 * testPoint(depthMap, pos, rot * vec2(  1.0,  0.0 ));
	vis += 0.75 * testPoint(depthMap, pos, rot * vec2(  0.0,  1.0 ));

	return vis / 4.0;
}

highp float testPoint_2_3(highp sampler2D depthMap, highp vec3 pos, highp mat2 rot)
{
	highp float vis = 0.0;
	vis += testPoint(depthMap, pos, rot * vec2( 0.1019, 0.0777 ));
	vis += testPoint(depthMap, pos, rot * vec2( -0.0029, -0.2387 ));
	vis += testPoint(depthMap, pos, rot * vec2( 0.6148, -0.0256 ));
	vis += testPoint(depthMap, pos, rot * vec2( 0.2746, 0.3408 ));
	vis += testPoint(depthMap, pos, rot * vec2( -0.2128, 0.2294 ));
	vis += testPoint(depthMap, pos, rot * vec2( -0.3600, -0.2485 ));
	vis += testPoint(depthMap, pos, rot * vec2( -0.0198, -0.6149 ));
	vis += testPoint(depthMap, pos, rot * vec2( 0.4676, -0.5034 ));
	vis += testPoint(depthMap, pos, rot * vec2( 0.7396, 0.4641 ));
	vis += testPoint(depthMap, pos, rot * vec2( 0.3454, 0.7955 ));
	vis += testPoint(depthMap, pos, rot * vec2( -0.1683, 0.8320 ));
	vis += testPoint(depthMap, pos, rot * vec2( -0.6054, 0.5596 ));
	vis += testPoint(depthMap, pos, rot * vec2( -0.7988, 0.0822 ));
	vis += testPoint(depthMap, pos, rot * vec2( -0.6747, -0.4176 ));
	vis += testPoint(depthMap, pos, rot * vec2( -0.2806, -0.7491 ));
	vis += testPoint(depthMap, pos, rot * vec2( 0.2332, -0.7856 ));
	vis += testPoint(depthMap, pos, rot * vec2( 0.6702, -0.5131 ));
	vis += testPoint(depthMap, pos, rot * vec2( 0.8637, -0.0358 ));

	return vis / 18.0;
}
