uniform lowp float ColorScale;
uniform lowp float AlphaScale;
uniform lowp float AmbientInfluence;
uniform lowp float MaxDiffuse;
uniform lowp vec4 TintColor;

#if defined( ENVIRONMENT_MAPPING )
uniform highp mat4 paraboloidViewMatrix;
#endif

uniform lowp vec3 lightDiffuse[MAX_LIGHTS];
uniform lowp vec3 lightPos[MAX_LIGHTS];

#ifndef AMBIENT_MAP
uniform lowp vec3 lightAmbient[MAX_LIGHTS];
#endif

#ifdef USE_SPEC
uniform lowp float SpecularPower;

	#if !defined( USE_SPECULAR_MAP )
	uniform lowp float SpecularIntensity;
	#endif

uniform lowp vec3 SpecularColor;
#endif

#ifdef USE_FOG
uniform lowp vec4 fogColor;
uniform lowp float FogInfluence;
#endif

#ifdef BURNING_MAP
uniform lowp float BurningThreshold;
uniform lowp vec3 FireColor;
#endif

#if defined( SHADOW_MAP )
uniform highp vec2 shadowMapSize;
uniform lowp float lightMaxShadow[MAX_LIGHTS];
#endif

#ifdef CHARACTER_LIGHT
uniform lowp float charLightIntensity;
uniform lowp float charLightMinRadius;
uniform lowp float charLightMaxRadius;
uniform lowp vec3 charLightColor;
uniform lowp vec3 charLightPos;
#endif

#ifdef RIM_LIGHT
uniform lowp float RimPower;
uniform lowp vec4 RimColor;
#endif

#if defined( SHADOW_BLOB )
uniform lowp vec3 ShadowBlobPositions[16];
#endif

#if defined( ENVIRONMENT_MAPPING )
	
	#if !defined( REFLECTANCE_MAP )
	uniform lowp float Reflectance;
	#endif

uniform lowp float ReflectionContrast;
#endif

#if defined( DETAIL_MAP )
uniform lowp float DetailNoiseIntensity;
uniform lowp float DetailNoisePower;
uniform lowp vec3 DetailNoiseRepeatFactors;
uniform lowp float DetailNoiseContrastFactor;
#endif

#ifdef PULSING_BRIGHTNESS
uniform highp float wrapTime;
uniform lowp vec4 PulsingColor;
uniform lowp float PulsingSpeed;
#endif

uniform sampler2D diffuseMap;

#ifdef USE_ALPHA_TEXTURE
uniform sampler2D alphaMap;
#endif

#ifdef USE_NM
uniform sampler2D normalMap;
#endif

#ifdef AMBIENT_MAP
uniform sampler2D ambientMap;
#endif

#if defined( USE_SPECULAR_MAP ) || defined( USE_GLOSS_MAP ) || defined( USE_SELF_ILLUM_MAP )
uniform sampler2D specularGlossSelfIllumMap;
#endif

#if defined( SHADOW_MAP )
uniform sampler2D shadowMap;
#endif

#if defined( SHADOW_BLOB )
uniform sampler2D shadowBlob;
#endif

#ifdef LIGHT_MAP
uniform sampler2D lightMap;
#endif

#ifdef BURNING_MAP
uniform sampler2D burningMap;
#endif

#ifdef ENVIRONMENT_MAPPING
uniform sampler2D environmentMap;
#endif

#if defined( ENVIRONMENT_MAPPING ) && defined( REFLECTANCE_MAP )
uniform sampler2D reflectanceMap;
#endif

#if defined( DETAIL_MAP )
uniform sampler2D detailMap;
uniform sampler2D detailNoiseMap;
#endif

#ifdef VC_CHANNEL_BLENDING
uniform sampler2D baseTexture;
uniform sampler2D redFactorTexture;
uniform sampler2D greenFactorTexture;
uniform sampler2D blueFactorTexture;
#endif		

#ifdef USE_NM
varying highp vec3 vNorm;
varying highp vec3 vTgt;
varying highp vec3 vBitgt;
#else
varying highp vec3 vNormal;
#endif

varying highp vec2 vUV0;

#if defined( USE_SPEC ) || defined( RIM_LIGHT ) || defined( ENVIRONMENT_MAPPING )
varying highp vec3 vEyeDir;
#endif

#ifdef USE_FOG
varying lowp float vFogFactor;
#endif

#if defined( SHADOW_MAP )
varying highp vec3 vPosShadowMapSpace;
#endif

#ifdef AMBIENT_MAP
varying lowp vec2 vPosAmbientMapSpace;
#endif

#if defined( CHARACTER_LIGHT ) || defined( SHADOW_BLOB )
varying highp vec3 vPosWS;
#endif

#ifdef CHARACTER_LIGHT
varying highp vec3 vCharLightDir;
#endif

#if defined( VERTEX_COLOR ) || defined( VC_CHANNEL_BLENDING )
varying lowp vec4 vColor;
#endif

#if defined( LIGHT_MAP ) || defined( USE_ALPHA_TEXTURE )
varying lowp vec2 vUV1;
#endif

#if defined( LIGHT_MAP ) || defined( ENVIRONMENT_MAPPING ) || defined( DETAIL_MAP )
const lowp vec3 k_colorWeights = vec3( 0.2126, 0.7152, 0.0722 );
#endif

#if  defined( SHADOW_BLOB )
const lowp float k_shadowBlobScaleFactor = 0.7;
#endif

#if defined( PULSING_BRIGHTNESS )
const highp float k_pi = 3.1415926;
const highp float k_2pi = k_pi * 2.0;
#endif

#if defined( ENVIRONMENT_MAPPING )
#include "paraboloid_project_uv.glsl"
#endif

#if defined( SHADOW_MAP )
	#if defined( ADVANCED_SHADOW_MAP )
		#include "pcf_advanced.glsl"
	#elif defined( HIGH_QUALITY_SHADOW_MAP )
		#include "pcf_high.glsl"
	#elif defined( NORMAL_QUALITY_SHADOW_MAP )
		#include "pcf_normal.glsl"
	#else
		#include "pcf_low.glsl"
	#endif
#endif
	
#if defined( SHADOW_BLOB )
	#if defined( ADVANCED_SHADOW_BLOB )
		#include "shadow_blob_advanced.glsl"
	#elif defined( HIGH_QUALITY_SHADOW_BLOB )
		#include "shadow_blob_high.glsl"
	#elif defined( NORMAL_QUALITY_SHADOW_BLOB )
		#include "shadow_blob_normal.glsl"
	#else
		#include "shadow_blob_low.glsl"
	#endif
#endif

void main()
{
#ifdef BURNING_MAP
	lowp float burningAlpha = texture2D( burningMap, vUV0 ).r;
	if( burningAlpha > BurningThreshold )
		discard;
#endif

#ifndef VC_CHANNEL_BLENDING
	lowp vec4 diffuseCol = jetAlphaTexture2D( diffuseMap, vUV0 ) * TintColor;
#else
	lowp vec4 colBase = texture2D( baseTexture, vUV0 );
	lowp vec4 colr = texture2D( redFactorTexture, vUV0 );
	lowp vec4 colg = texture2D( greenFactorTexture, vUV0 );
	lowp vec4 colb = texture2D( blueFactorTexture, vUV0 );
	
	lowp float baseFactor = 1.0 - min( vColor.r + vColor.g + vColor.b, 1.0 );
	lowp vec4 diffuseCol = baseFactor * colBase + colr * vColor.r + colg * vColor.g + colb * vColor.b;
#endif
	
#ifdef USE_ALPHA_TEXTURE
	diffuseCol.a *= texture2D( alphaMap, vUV1 ).r;
#endif

#ifdef USE_SELF_ILLUM_MAP
	lowp vec4 diffuseTexCol = diffuseCol;
#endif

#ifdef DETAIL_MAP
	lowp vec4 detailCol4 = texture2D( detailMap, vUV0 );
	lowp vec3 detailCol = detailCol4.rgb * detailCol4.a;
	
	lowp float detailNoise1 = texture2D( detailNoiseMap, vUV0 * DetailNoiseRepeatFactors.x ).r;
	lowp float detailNoise2 = texture2D( detailNoiseMap, vUV0 * DetailNoiseRepeatFactors.y ).r;
	lowp float detailNoise3 = texture2D( detailNoiseMap, vUV0 * DetailNoiseRepeatFactors.z ).r;
	
	lowp float detailNoise = max( max( detailNoise1, detailNoise2 ), detailNoise3 );
	lowp float adaptedDetailNoise = min( 1.0, pow( detailNoise + DetailNoiseIntensity, DetailNoisePower ) );

	detailCol *= adaptedDetailNoise;
	diffuseCol = vec4( mix( diffuseCol.rgb, detailCol, min( 1.0, dot( detailCol, k_colorWeights ) * DetailNoiseContrastFactor ) ), 1.0 );
#endif
	
#ifdef VERTEX_COLOR
	diffuseCol *= vColor;
#endif
	
#ifdef USE_NM
	highp vec3 tgt = normalize( vTgt );
	highp vec3 bitgt = normalize( vBitgt );
	highp vec3 norm = normalize( vNorm );
	
	highp vec3 normal = mat3( tgt, bitgt, norm ) * ( texture2D( normalMap, vUV0 ).xyz * 2.0 - 1.0 );
#else
	highp vec3 normal = normalize( vNormal );
#endif
	
	lowp vec3 lightDir = normalize( lightPos[0] );
	lowp float NdotL = dot( normal, lightDir );
	
#ifdef LIGHT_MAP
	lowp vec3 lightMapCol = texture2D( lightMap, vUV1 ).rgb;
#else
	lowp float diffuseIntensity = max( NdotL, 0.0 );
#endif

#if defined( SHADOW_MAP ) || defined( SHADOW_BLOB )
	lowp float shadowIntensity = 1.0;
#endif
	
#if defined( SHADOW_MAP )
	bool frontFacedToLight = ( NdotL >= 0.0 ); 
	bvec4 edgesCorrect = bvec4( vPosShadowMapSpace.x >= 0.0, vPosShadowMapSpace.x < 1.0, vPosShadowMapSpace.y >= 0.0, vPosShadowMapSpace.y < 1.0 );
	lowp float shadowValue = frontFacedToLight && all( edgesCorrect ) ? texture2DPCF( shadowMap, shadowMapSize, vPosShadowMapSpace.xy, vPosShadowMapSpace.z ) : 1.0;

	#ifdef LIGHT_MAP
		lowp float maxShadow = lightMaxShadow[0] * 0.5;
	#else
		lowp float maxShadow = lightMaxShadow[0];
	#endif

	shadowIntensity = mix( 1.0, shadowValue, maxShadow );
#endif
	
#if defined( SHADOW_BLOB )
	shadowIntensity = min( shadowIntensity, texture2DShadowBlob( shadowBlob, vPosWS, k_shadowBlobScaleFactor ) );
#endif

#if defined( SHADOW_MAP ) || defined( SHADOW_BLOB )
	#ifdef LIGHT_MAP
		lightMapCol *= shadowIntensity;
	#else
		diffuseIntensity = min( diffuseIntensity, shadowIntensity );
	#endif
#endif

#ifdef CHARACTER_LIGHT
	lowp float charLightDist = length( charLightPos - vPosWS );
	lowp vec3 charLightDir = normalize( vCharLightDir );
	lowp float charLightFactor = ( charLightDist - charLightMinRadius ) / ( charLightMaxRadius - charLightMinRadius );
	charLightFactor = 1.0 - clamp( charLightFactor, 0.0, 1.0 );
	
	lowp float charLightNdotL = dot( normal, charLightDir );
	lowp vec3 charLightFinalColor = charLightColor * max( charLightNdotL, 0.0 ) * charLightFactor * charLightIntensity;
#endif

#ifdef AMBIENT_MAP
	lowp vec3 lightAmb = texture2D( ambientMap, vPosAmbientMapSpace ).rgb;
#else
	lowp vec3 lightAmb = lightAmbient[0];
#endif

#ifdef LIGHT_MAP
	lowp float diffuseIntensity = dot( lightMapCol, k_colorWeights );	//	approximate diffuse intensity
#endif

	diffuseIntensity *= MaxDiffuse;
	lowp float lightDiffuseFactor = mix( 1.0, diffuseIntensity, AmbientInfluence );
	
#ifdef LIGHT_MAP
	diffuseCol.rgb *= mix( lightAmb, lightMapCol, lightDiffuseFactor );
#endif
	
	lowp vec3 color = ( diffuseCol.rgb * mix( lightAmb, lightDiffuse[0], lightDiffuseFactor ) ) * ColorScale;
	
	
#if defined( USE_SPECULAR_MAP ) || defined( USE_GLOSS_MAP ) || defined( USE_SELF_ILLUM_MAP )
	lowp vec3 specGlossSi = texture2D( specularGlossSelfIllumMap, vUV0 ).rgb;
#endif
	
#ifdef USE_SPEC
	lowp vec3 specColor = vec3( 0.0, 0.0, 0.0 );
	if( NdotL > 0.0 )
	{
		lowp vec3 reflectDir = reflect( -lightDir, normal );
		lowp float factor = max( dot( normalize( vEyeDir ), reflectDir ), 0.0 );
	
	#if defined( USE_SPECULAR_MAP )
		lowp float specIntensity = specGlossSi.r;
	#else
		lowp float specIntensity = SpecularIntensity;
	#endif
	
	#if defined( USE_GLOSS_MAP )
		lowp float specPow = specGlossSi.g * SpecularPower;
	#else
		lowp float specPow = SpecularPower;
	#endif
	
		specColor = max( SpecularColor * specIntensity * pow( factor, specPow ), 0.0 );
	}
	
	color += specColor;
#endif

#ifdef ENVIRONMENT_MAPPING
	lowp vec3 reflectDir = reflect( normalize( -vEyeDir ), normal );
	lowp vec2 reflectUV = ParaboloidProjectUV( vec3( paraboloidViewMatrix * vec4( reflectDir, 0.0 ) ) );
	lowp vec3 envMapCol = texture2D( environmentMap, reflectUV ).rgb;
	lowp float reflectBrightness = dot( envMapCol, k_colorWeights );
	lowp vec3 reflectCol = mix( vec3( reflectBrightness ), envMapCol, ReflectionContrast );
	
	#ifdef REFLECTANCE_MAP
		color = mix( color, reflectCol, texture2D( reflectanceMap, vUV0 ).r );
	#else
		color = mix( color, reflectCol, Reflectance );
	#endif
#endif
	
#ifdef CHARACTER_LIGHT
	color += diffuseCol.rgb * charLightFinalColor;
#endif

#ifdef RIM_LIGHT
	lowp float fresnel = 1.0 - max( dot( normalize( vEyeDir ), normal ), 0.0 );
	color = mix( color, RimColor.rgb, pow( fresnel, RimPower ) * RimColor.a );
#endif

#ifdef USE_SELF_ILLUM_MAP
	color = mix( color, diffuseTexCol.rgb, specGlossSi.b );
#endif

#ifdef PULSING_BRIGHTNESS
	// clamp driver wraptime to 0..2pi
	highp float factor = ( wrapTime / 30.0 ) * k_2pi;
	highp float pulsingSpeed = floor( PulsingSpeed ) * 3.0;		// adjusting pulsing speed (will fix this in future via clara parameters)
	color += PulsingColor.rgb * ( sin( factor * pulsingSpeed ) + 1.0 ) * 0.5;
#endif

#ifdef USE_FOG	
	color = mix( color, fogColor.rgb, vFogFactor * FogInfluence );
#endif
	
	lowp vec4 finalColor = vec4( color, diffuseCol.a * AlphaScale );
	
#ifdef BURNING_MAP
	lowp float fireFactor = BurningThreshold - burningAlpha;
	lowp float burningMixFactor = max( 0.1 - fireFactor, 0.0 ) * 10.0;
	finalColor.rgb = mix( finalColor.rgb, FireColor, burningMixFactor );
#endif	
	
	gl_FragColor = finalColor;
}
