uniform highp mat4 worldMatrix;
uniform highp mat4 worldViewProjMatrix;

#if defined( USE_FOG ) || defined( DUAL_PARABOLOID_PROJECTION )
uniform highp mat4 worldViewMatrix;
#endif

#ifdef AMBIENT_MAP
uniform highp mat4 ambientMapVPMatrix;
#endif

#if defined( USE_SPEC ) || defined( RIM_LIGHT ) || defined( ENVIRONMENT_MAPPING )
uniform lowp vec3 cameraPos;
#endif

#ifdef USE_FOG
uniform lowp vec4 fogColor;
uniform lowp float fogStart;
uniform lowp float fogRangeI;
#endif

#if defined( DUAL_PARABOLOID_PROJECTION )
uniform lowp float cameraFar;
#endif

#if defined( SHADOW_MAP )
uniform highp mat4 shadowMapVPMatrix;
#endif

#ifdef CHARACTER_LIGHT
uniform lowp vec3 charLightPos;
#endif

uniform lowp vec2 DiffuseMapUVScale;
uniform lowp vec2 DiffuseMapUVOffset;

attribute vec4 inPos;
attribute vec4 inNormal;
attribute vec2 inUV0;

#ifdef USE_NM
attribute vec4 inTangent;
#endif

#if defined( LIGHT_MAP ) || defined( USE_ALPHA_TEXTURE )
attribute vec2 inUV1;
#endif

#if defined( VERTEX_COLOR ) || defined( VC_CHANNEL_BLENDING )
attribute vec4 inColor;
#endif

#ifdef USE_NM
varying highp vec3 vNorm;
varying highp vec3 vTgt;
varying highp vec3 vBitgt;
#else
varying highp vec3 vNormal;
#endif

varying highp vec3 vLightDir;
varying highp vec2 vUV0;

#if defined( USE_SPEC ) || defined( RIM_LIGHT ) || defined( ENVIRONMENT_MAPPING )
varying highp vec3 vEyeDir;
#endif

#ifdef USE_FOG
varying lowp float vFogFactor;
#endif

#if defined( SHADOW_MAP )
varying highp vec3 vPosShadowMapSpace;
#endif

#ifdef AMBIENT_MAP
varying lowp vec2 vPosAmbientMapSpace;
#endif

#if defined( CHARACTER_LIGHT ) || defined( SHADOW_BLOB )
varying highp vec3 vPosWS;
#endif

#ifdef CHARACTER_LIGHT
varying highp vec3 vCharLightDir;
#endif

#if defined( VERTEX_COLOR ) || defined( VC_CHANNEL_BLENDING )
varying lowp vec4 vColor;
#endif

#if defined( LIGHT_MAP ) || defined( USE_ALPHA_TEXTURE )
varying lowp vec2 vUV1;
#endif

#if defined( DUAL_PARABOLOID_PROJECTION )
#include "paraboloid_project.glsl"
#endif


void main()
{	
	highp vec4 posOS = vec4( inPos.xyz, 1.0 );
	highp vec4 posWS = worldMatrix * posOS;
	
#ifdef USE_NM
	highp vec3 norm = inNormal.xyz;
	highp vec3 tgt = inTangent.xyz * inTangent.w;	// for tangents w coordinate may contain handness (checkout collada2pig sources)
	highp vec3 bitgt = cross( norm, tgt );
	
	highp mat3 tbn = mat3( worldMatrix[0].xyz, worldMatrix[1].xyz, worldMatrix[2].xyz ) * mat3( tgt, bitgt, norm );
	
	vTgt = tbn[0];
	vBitgt = tbn[1];
	vNorm = tbn[2];
#else
	vNormal = vec3( worldMatrix * vec4( inNormal.xyz, 0.0 ) );
#endif
	
	vUV0 = inUV0 * DiffuseMapUVScale + DiffuseMapUVOffset;
	
#if defined( LIGHT_MAP ) || defined( USE_ALPHA_TEXTURE )
	vUV1 = inUV1;
#endif
	
#if defined( VERTEX_COLOR ) || defined( VC_CHANNEL_BLENDING )
	vColor = inColor;
#endif
	
#if defined( USE_SPEC ) || defined( RIM_LIGHT ) || defined( ENVIRONMENT_MAPPING )
	vEyeDir = cameraPos - posWS.xyz;
#endif
	
#ifdef USE_FOG
	lowp float vertDepth = -( worldViewMatrix * posOS ).z;
	lowp float fog = clamp( ( vertDepth - fogStart ) * fogRangeI, 0.0, 1.0 );
	vFogFactor = fog * fogColor.a;
#endif

#if defined( SHADOW_MAP )
	vPosShadowMapSpace = vec3( shadowMapVPMatrix * posWS );
#endif

#ifdef AMBIENT_MAP
	vPosAmbientMapSpace = vec2( ambientMapVPMatrix * posWS );
#endif
	
#if defined( CHARACTER_LIGHT ) || defined( SHADOW_BLOB )
	vPosWS = posWS.xyz;
#endif

#ifdef CHARACTER_LIGHT
	vCharLightDir = charLightPos - vPosWS;
#endif

#ifndef DUAL_PARABOLOID_PROJECTION
	gl_Position = worldViewProjMatrix * posOS;
#else
	gl_Position = ParaboloidProject( worldViewMatrix * posOS, cameraFar );
#endif
}