//
// THIS FILE IS GENERATED AUTOMATICALLY, DO NOT CHANGE IT BY HAND!
//


uniform lowp float ColorScale;
uniform lowp float AlphaScale;
uniform lowp float AmbientInfluence;
uniform lowp float MaxDiffuse;
uniform lowp vec4 TintColor;

uniform lowp vec3 lightDiffuse[MAX_LIGHTS];
uniform lowp vec3 lightPos[MAX_LIGHTS];

uniform lowp vec3 lightAmbient[MAX_LIGHTS];

#ifdef USE_FOG
uniform mediump vec4 fogColor;
uniform lowp float FogInfluence;
#endif

#ifdef SHADOW_MAP
uniform highp vec2 shadowMapSize;
uniform lowp float lightMaxShadow[MAX_LIGHTS];
#endif

#ifdef CHARACTER_LIGHT
uniform lowp float charLightIntensity;
uniform lowp float charLightMinRadius;
uniform lowp float charLightMaxRadius;
uniform lowp vec3 charLightColor;
uniform lowp vec3 charLightPos;
uniform	lowp float CharacterLightValue;
#endif

#ifdef PULSING_BRIGHTNESS
uniform highp float wrapTime;
uniform lowp vec4 PulsingColor;
uniform lowp float PulsingSpeed;
#endif

#ifdef ADDITIONAL_COLOR
uniform lowp vec3 AdditionalColor;
#endif

uniform sampler2D diffuseMap;

#ifdef SHADOW_MAP
uniform sampler2D shadowMap;
#endif

varying highp vec3 vNormal;

varying highp vec2 vUV0;

#ifdef USE_FOG
varying lowp float vFogFactor;
#endif

#ifdef SHADOW_MAP
varying highp vec3 vPosShadowMapSpace;
#endif

#ifdef CHARACTER_LIGHT
	#ifndef USE_LOWP_POSWS
varying highp vec3 vPosWS;
	#else
varying lowp vec3 vPosWS;
	#endif
#endif

#ifdef CHARACTER_LIGHT
varying highp vec3 vCharLightDir;
#endif

#ifdef PULSING_BRIGHTNESS
const highp float k_pi = 3.1415926;
const highp float k_2pi = k_pi * 2.0;
#endif

#ifdef SHADOW_MAP
	#ifdef ADVANCED_SHADOW_MAP
		#include "pcf_advanced.glsl"
	#elif defined( HIGH_QUALITY_SHADOW_MAP )
		#include "pcf_high.glsl"
	#elif defined( NORMAL_QUALITY_SHADOW_MAP )
		#include "pcf_normal.glsl"
	#else
		#include "pcf_low.glsl"
	#endif
#endif

void main()
{

		lowp vec4 diffuseCol = jetAlphaTexture2D( diffuseMap, vUV0 ) * TintColor;

	highp vec3 normal = normalize( vNormal );
	
	lowp vec3 lightDir = normalize( lightPos[0] );
	lowp float NdotL = dot( normal, lightDir );
	
	lowp float diffuseIntensity = max( NdotL, 0.0 );

#ifdef SHADOW_MAP
	lowp float shadowIntensity = 1.0;
#endif
	
#ifdef SHADOW_MAP
	bool frontFacedToLight = ( NdotL >= 0.0 ); 
	bvec4 edgesCorrect = bvec4( vPosShadowMapSpace.x >= 0.0, vPosShadowMapSpace.x < 1.0, vPosShadowMapSpace.y >= 0.0, vPosShadowMapSpace.y < 1.0 );
	lowp float shadowValue = frontFacedToLight && all( edgesCorrect ) ? texture2DPCF( shadowMap, shadowMapSize, vPosShadowMapSpace.xy, vPosShadowMapSpace.z ) : 1.0;

		lowp float maxShadow = lightMaxShadow[0];

	shadowIntensity = mix( 1.0, shadowValue, maxShadow );
#endif

#ifdef SHADOW_MAP
		diffuseCol *= shadowIntensity;
#endif

#ifdef CHARACTER_LIGHT
	lowp float charLightDist = length( charLightPos - vPosWS );
	lowp vec3 charLightDir = normalize( vCharLightDir );
	lowp float charLightFactor = ( charLightDist - charLightMinRadius ) / ( charLightMaxRadius - charLightMinRadius );
	charLightFactor = 1.0 - clamp( charLightFactor, 0.0, 1.0 );
	
	lowp float charLightNdotL = dot( normal, charLightDir );
	lowp vec3 charLightFinalColor = charLightColor * max( charLightNdotL, 0.0 ) * charLightFactor * charLightIntensity;
#endif

	lowp vec3 lightAmb = lightAmbient[0];

	diffuseIntensity *= MaxDiffuse;
	lowp float lightDiffuseFactor = mix( 1.0, diffuseIntensity, AmbientInfluence );
	
	lowp vec3 color = ( diffuseCol.rgb * mix( lightAmb, lightDiffuse[0], lightDiffuseFactor ) ) * ColorScale;
	
#ifdef CHARACTER_LIGHT
	color += diffuseCol.rgb * charLightFinalColor * CharacterLightValue;
#endif

#ifdef PULSING_BRIGHTNESS
	// clamp driver wraptime to 0..2pi
	highp float factor = ( wrapTime / 30.0 ) * k_2pi;
	highp float pulsingSpeed = floor( PulsingSpeed ) * 3.0;		// adjusting pulsing speed (will fix this in future via clara parameters)
	color += PulsingColor.rgb * ( sin( factor * pulsingSpeed ) + 1.0 ) * 0.5;
#endif

#ifdef ADDITIONAL_COLOR
	color += AdditionalColor.rgb;
#endif

#ifdef USE_FOG
	color = mix( color, fogColor.rgb, vFogFactor * FogInfluence );
#endif
	
	lowp vec4 finalColor = vec4( color, diffuseCol.a * AlphaScale );
	
	gl_FragColor = finalColor;
}
