//
// THIS FILE IS GENERATED AUTOMATICALLY, DO NOT CHANGE IT BY HAND!
//


uniform lowp float ColorScale;
uniform lowp float AlphaScale;
uniform lowp float AmbientInfluence;
uniform lowp float MaxDiffuse;
uniform lowp vec4 TintColor;

uniform lowp vec3 lightDiffuse[MAX_LIGHTS];
uniform lowp vec3 lightPos[MAX_LIGHTS];

uniform lowp vec3 lightAmbient[MAX_LIGHTS];

#ifdef USE_FOG
uniform mediump vec4 fogColor;
uniform lowp float FogInfluence;
#endif

#ifdef SHADOW_MAP
uniform highp vec2 shadowMapSize;
uniform lowp float lightMaxShadow[MAX_LIGHTS];
#endif

#ifdef CHARACTER_LIGHT
uniform lowp float charLightIntensity;
uniform lowp float charLightMinRadius;
uniform lowp float charLightMaxRadius;
uniform lowp vec3 charLightColor;
uniform lowp vec3 charLightPos;
uniform	lowp float CharacterLightValue;
#endif

#ifdef DETAIL_MAP
uniform lowp float DetailNoiseIntensity;
uniform lowp float DetailNoisePower;
uniform lowp vec3 DetailNoiseRepeatFactors;
uniform lowp float DetailNoiseContrastFactor;
#endif

uniform sampler2D diffuseMap;

#ifdef SHADOW_MAP
uniform sampler2D shadowMap;
#endif

#ifdef DETAIL_MAP
uniform sampler2D detailMap;
uniform sampler2D detailNoiseMap;
#endif

varying highp vec3 vNormal;

varying highp vec2 vUV0;

#ifdef USE_FOG
varying lowp float vFogFactor;
#endif

#ifdef SHADOW_MAP
varying highp vec3 vPosShadowMapSpace;
#endif

#ifdef CHARACTER_LIGHT
	#ifndef USE_LOWP_POSWS
varying highp vec3 vPosWS;
	#else
varying lowp vec3 vPosWS;
	#endif
#endif

#ifdef CHARACTER_LIGHT
varying highp vec3 vCharLightDir;
#endif

#ifdef DETAIL_MAP
const lowp vec3 k_colorWeights = vec3( 0.2126, 0.7152, 0.0722 );
#endif

#ifdef SHADOW_MAP
	#ifdef ADVANCED_SHADOW_MAP
		#include "pcf_advanced.glsl"
	#elif defined( HIGH_QUALITY_SHADOW_MAP )
		#include "pcf_high.glsl"
	#elif defined( NORMAL_QUALITY_SHADOW_MAP )
		#include "pcf_normal.glsl"
	#else
		#include "pcf_low.glsl"
	#endif
#endif

void main()
{

		lowp vec4 diffuseCol = jetAlphaTexture2D( diffuseMap, vUV0 ) * TintColor;

#ifdef DETAIL_MAP
	lowp vec4 detailCol4 = texture2D( detailMap, vUV0 );
	lowp vec3 detailCol = detailCol4.rgb * detailCol4.a;
	
	lowp float detailNoise1 = texture2D( detailNoiseMap, vUV0 * DetailNoiseRepeatFactors.x ).r;
	lowp float detailNoise2 = texture2D( detailNoiseMap, vUV0 * DetailNoiseRepeatFactors.y ).r;
	lowp float detailNoise3 = texture2D( detailNoiseMap, vUV0 * DetailNoiseRepeatFactors.z ).r;
	
	lowp float detailNoise = max( max( detailNoise1, detailNoise2 ), detailNoise3 );
	lowp float adaptedDetailNoise = min( 1.0, pow( detailNoise + DetailNoiseIntensity, DetailNoisePower ) );

	detailCol *= adaptedDetailNoise;
	diffuseCol = vec4( mix( diffuseCol.rgb, detailCol, min( 1.0, dot( detailCol, k_colorWeights ) * DetailNoiseContrastFactor ) ), 1.0 );
#endif
	
	highp vec3 normal = normalize( vNormal );
	
	lowp vec3 lightDir = normalize( lightPos[0] );
	lowp float NdotL = dot( normal, lightDir );
	
	lowp float diffuseIntensity = max( NdotL, 0.0 );

#ifdef SHADOW_MAP
	lowp float shadowIntensity = 1.0;
#endif
	
#ifdef SHADOW_MAP
	bool frontFacedToLight = ( NdotL >= 0.0 ); 
	bvec4 edgesCorrect = bvec4( vPosShadowMapSpace.x >= 0.0, vPosShadowMapSpace.x < 1.0, vPosShadowMapSpace.y >= 0.0, vPosShadowMapSpace.y < 1.0 );
	lowp float shadowValue = frontFacedToLight && all( edgesCorrect ) ? texture2DPCF( shadowMap, shadowMapSize, vPosShadowMapSpace.xy, vPosShadowMapSpace.z ) : 1.0;

		lowp float maxShadow = lightMaxShadow[0];

	shadowIntensity = mix( 1.0, shadowValue, maxShadow );
#endif

#ifdef SHADOW_MAP
		diffuseCol *= shadowIntensity;
#endif

#ifdef CHARACTER_LIGHT
	lowp float charLightDist = length( charLightPos - vPosWS );
	lowp vec3 charLightDir = normalize( vCharLightDir );
	lowp float charLightFactor = ( charLightDist - charLightMinRadius ) / ( charLightMaxRadius - charLightMinRadius );
	charLightFactor = 1.0 - clamp( charLightFactor, 0.0, 1.0 );
	
	lowp float charLightNdotL = dot( normal, charLightDir );
	lowp vec3 charLightFinalColor = charLightColor * max( charLightNdotL, 0.0 ) * charLightFactor * charLightIntensity;
#endif

	lowp vec3 lightAmb = lightAmbient[0];

	diffuseIntensity *= MaxDiffuse;
	lowp float lightDiffuseFactor = mix( 1.0, diffuseIntensity, AmbientInfluence );
	
	lowp vec3 color = ( diffuseCol.rgb * mix( lightAmb, lightDiffuse[0], lightDiffuseFactor ) ) * ColorScale;
	
#ifdef CHARACTER_LIGHT
	color += diffuseCol.rgb * charLightFinalColor * CharacterLightValue;
#endif

#ifdef USE_FOG
	color = mix( color, fogColor.rgb, vFogFactor * FogInfluence );
#endif
	
	lowp vec4 finalColor = vec4( color, diffuseCol.a * AlphaScale );
	
	gl_FragColor = finalColor;
}
