//
// THIS FILE IS GENERATED AUTOMATICALLY, DO NOT CHANGE IT BY HAND!
//


uniform lowp float ColorScale;
uniform lowp float AlphaScale;
uniform lowp float AmbientInfluence;
uniform lowp float MaxDiffuse;
uniform lowp vec4 TintColor;

uniform lowp vec3 lightDiffuse[MAX_LIGHTS];
uniform lowp vec3 lightPos[MAX_LIGHTS];

#ifndef AMBIENT_MAP
uniform lowp vec3 lightAmbient[MAX_LIGHTS];
#endif

#ifdef USE_SPEC
uniform lowp float SpecularPower;

	uniform lowp float SpecularIntensity;

uniform lowp vec3 SpecularColor;
#endif

#ifdef USE_FOG
uniform lowp vec4 fogColor;
uniform lowp float FogInfluence;
#endif

#ifdef SHADOW_MAP
uniform highp vec2 shadowMapSize;
uniform lowp float lightMaxShadow[MAX_LIGHTS];
#endif

#ifdef CHARACTER_LIGHT
uniform lowp float charLightIntensity;
uniform lowp float charLightMinRadius;
uniform lowp float charLightMaxRadius;
uniform lowp vec3 charLightColor;
uniform lowp vec3 charLightPos;
#endif

#ifdef SHADOW_BLOB
uniform lowp vec3 ShadowBlobPositions[16];
#endif

uniform sampler2D diffuseMap;

#ifdef USE_NM
uniform sampler2D normalMap;
#endif

#ifdef AMBIENT_MAP
uniform sampler2D ambientMap;
#endif

#ifdef SHADOW_MAP
uniform sampler2D shadowMap;
#endif

#ifdef SHADOW_BLOB
uniform sampler2D shadowBlob;
#endif

#ifdef USE_NM
varying highp vec3 vNorm;
varying highp vec3 vTgt;
varying highp vec3 vBitgt;
#else
varying highp vec3 vNormal;
#endif

varying highp vec2 vUV0;

#ifdef USE_SPEC
varying highp vec3 vEyeDir;
#endif

#ifdef USE_FOG
varying lowp float vFogFactor;
#endif

#ifdef SHADOW_MAP
varying highp vec3 vPosShadowMapSpace;
#endif

#ifdef AMBIENT_MAP
varying lowp vec2 vPosAmbientMapSpace;
#endif

#if defined( CHARACTER_LIGHT ) || defined( SHADOW_BLOB )
varying highp vec3 vPosWS;
#endif

#ifdef CHARACTER_LIGHT
varying highp vec3 vCharLightDir;
#endif

#ifdef SHADOW_BLOB
const lowp float k_shadowBlobScaleFactor = 0.7;
#endif

#ifdef SHADOW_MAP
	#ifdef ADVANCED_SHADOW_MAP
		#include "pcf_advanced.glsl"
	#elif defined( HIGH_QUALITY_SHADOW_MAP )
		#include "pcf_high.glsl"
	#elif defined( NORMAL_QUALITY_SHADOW_MAP )
		#include "pcf_normal.glsl"
	#else
		#include "pcf_low.glsl"
	#endif
#endif
	
#ifdef SHADOW_BLOB
	#ifdef ADVANCED_SHADOW_BLOB
		#include "shadow_blob_advanced.glsl"
	#elif defined( HIGH_QUALITY_SHADOW_BLOB )
		#include "shadow_blob_high.glsl"
	#elif defined( NORMAL_QUALITY_SHADOW_BLOB )
		#include "shadow_blob_normal.glsl"
	#else
		#include "shadow_blob_low.glsl"
	#endif
#endif

void main()
{

	lowp vec4 diffuseCol = jetAlphaTexture2D( diffuseMap, vUV0 ) * TintColor;
	
#ifdef USE_NM
	highp vec3 tgt = normalize( vTgt );
	highp vec3 bitgt = normalize( vBitgt );
	highp vec3 norm = normalize( vNorm );
	
	highp vec3 normal = mat3( tgt, bitgt, norm ) * ( texture2D( normalMap, vUV0 ).xyz * 2.0 - 1.0 );
#else
	highp vec3 normal = normalize( vNormal );
#endif
	
	lowp vec3 lightDir = normalize( lightPos[0] );
	lowp float NdotL = dot( normal, lightDir );
	
	lowp float diffuseIntensity = max( NdotL, 0.0 );

#if defined( SHADOW_MAP ) || defined( SHADOW_BLOB )
	lowp float shadowIntensity = 1.0;
#endif
	
#ifdef SHADOW_MAP
	bool frontFacedToLight = ( NdotL >= 0.0 ); 
	bvec4 edgesCorrect = bvec4( vPosShadowMapSpace.x >= 0.0, vPosShadowMapSpace.x < 1.0, vPosShadowMapSpace.y >= 0.0, vPosShadowMapSpace.y < 1.0 );
	lowp float shadowValue = frontFacedToLight && all( edgesCorrect ) ? texture2DPCF( shadowMap, shadowMapSize, vPosShadowMapSpace.xy, vPosShadowMapSpace.z ) : 1.0;

		lowp float maxShadow = lightMaxShadow[0];

	shadowIntensity = mix( 1.0, shadowValue, maxShadow );
#endif
	
#ifdef SHADOW_BLOB
	shadowIntensity = min( shadowIntensity, texture2DShadowBlob( shadowBlob, vPosWS, k_shadowBlobScaleFactor ) );
#endif

#if defined( SHADOW_MAP ) || defined( SHADOW_BLOB )
		diffuseIntensity = min( diffuseIntensity, shadowIntensity );
#endif

#ifdef CHARACTER_LIGHT
	lowp float charLightDist = length( charLightPos - vPosWS );
	lowp vec3 charLightDir = normalize( vCharLightDir );
	lowp float charLightFactor = ( charLightDist - charLightMinRadius ) / ( charLightMaxRadius - charLightMinRadius );
	charLightFactor = 1.0 - clamp( charLightFactor, 0.0, 1.0 );
	
	lowp float charLightNdotL = dot( normal, charLightDir );
	lowp vec3 charLightFinalColor = charLightColor * max( charLightNdotL, 0.0 ) * charLightFactor * charLightIntensity;
#endif

#ifdef AMBIENT_MAP
	lowp vec3 lightAmb = texture2D( ambientMap, vPosAmbientMapSpace ).rgb;
#else
	lowp vec3 lightAmb = lightAmbient[0];
#endif

	diffuseIntensity *= MaxDiffuse;
	lowp float lightDiffuseFactor = mix( 1.0, diffuseIntensity, AmbientInfluence );
	
	lowp vec3 color = ( diffuseCol.rgb * mix( lightAmb, lightDiffuse[0], lightDiffuseFactor ) ) * ColorScale;
	
#ifdef USE_SPEC
	lowp vec3 specColor = vec3( 0.0, 0.0, 0.0 );
	if( NdotL > 0.0 )
	{
		lowp vec3 reflectDir = reflect( -lightDir, normal );
		lowp float factor = max( dot( normalize( vEyeDir ), reflectDir ), 0.0 );
	
		lowp float specIntensity = SpecularIntensity;
	
		lowp float specPow = SpecularPower;
	
		specColor = max( SpecularColor * specIntensity * pow( factor, specPow ), 0.0 );
	}
	
	color += specColor;
#endif

#ifdef CHARACTER_LIGHT
	color += diffuseCol.rgb * charLightFinalColor;
#endif

#ifdef USE_FOG
	color = mix( color, fogColor.rgb, vFogFactor * FogInfluence );
#endif
	
	lowp vec4 finalColor = vec4( color, diffuseCol.a * AlphaScale );
	
	gl_FragColor = finalColor;
}
