//
// THIS FILE IS GENERATED AUTOMATICALLY, DO NOT CHANGE IT BY HAND!
//


uniform lowp float ColorScale;
uniform lowp float AlphaScale;
uniform lowp float AmbientInfluence;
uniform lowp float MaxDiffuse;
uniform lowp vec4 TintColor;

uniform lowp vec3 lightDiffuse[MAX_LIGHTS];
uniform lowp vec3 lightPos[MAX_LIGHTS];

uniform mediump vec2 gReflectionParams;

#ifndef AMBIENT_MAP
uniform lowp vec3 lightAmbient[MAX_LIGHTS];
#endif

#ifdef USE_FOG
uniform lowp vec4 fogColor;
uniform lowp float FogInfluence;
#endif

#ifdef SHADOW_MAP
uniform highp vec2 shadowMapSize;
uniform lowp float lightMaxShadow[MAX_LIGHTS];
#endif

#ifdef CHARACTER_LIGHT
uniform lowp float charLightIntensity;
uniform lowp float charLightMinRadius;
uniform lowp float charLightMaxRadius;
uniform lowp vec3 charLightColor;
uniform lowp vec3 charLightPos;
#endif

#ifdef SHADOW_BLOB
uniform lowp vec3 ShadowBlobPositions[16];
#endif

uniform sampler2D diffuseMap;

#ifdef AMBIENT_MAP
uniform sampler2D ambientMap;
#endif

#ifdef SHADOW_MAP
uniform sampler2D shadowMap;
#endif

#ifdef SHADOW_BLOB
uniform sampler2D shadowBlob;
#endif

varying highp vec3 vNormal;

varying highp vec2 vUV0;

#ifdef USE_FOG
varying lowp float vFogFactor;
#endif

#ifdef SHADOW_MAP
varying highp vec3 vPosShadowMapSpace;
#endif

#ifdef AMBIENT_MAP
varying lowp vec2 vPosAmbientMapSpace;
#endif

varying highp vec3 vPosWS;


#ifdef CHARACTER_LIGHT
varying highp vec3 vCharLightDir;
#endif

#ifdef VERTEX_COLOR
varying lowp vec4 vColor;
#endif

#ifdef SHADOW_BLOB
const lowp float k_shadowBlobScaleFactor = 0.7;
#endif

#ifdef SHADOW_MAP
	#ifdef ADVANCED_SHADOW_MAP
		#include "pcf_advanced.glsl"
	#elif defined( HIGH_QUALITY_SHADOW_MAP )
		#include "pcf_high.glsl"
	#elif defined( NORMAL_QUALITY_SHADOW_MAP )
		#include "pcf_normal.glsl"
	#else
		#include "pcf_low.glsl"
	#endif
#endif
	
#ifdef SHADOW_BLOB
	#ifdef ADVANCED_SHADOW_BLOB
		#include "shadow_blob_advanced.glsl"
	#elif defined( HIGH_QUALITY_SHADOW_BLOB )
		#include "shadow_blob_high.glsl"
	#elif defined( NORMAL_QUALITY_SHADOW_BLOB )
		#include "shadow_blob_normal.glsl"
	#else
		#include "shadow_blob_low.glsl"
	#endif
#endif

highp float exp_falloff(highp vec3 nrm_direction, highp vec3 pos, highp float distance, highp float scale)
{
	highp float fdistance = dot(nrm_direction,pos);//distance on direction vector
	fdistance = min(distance,fdistance);//clamp to max fdistance->[-inf,distance]
	fdistance = fdistance - distance;//[-inf to 0]
	return exp (fdistance * scale);//(fdistance < 0) -> exp -> (0,1]
}



void main()
{

	lowp vec4 diffuseCol = jetAlphaTexture2D( diffuseMap, vUV0 ) * TintColor;
	
#ifdef VERTEX_COLOR
	diffuseCol *= vColor;
#endif
	
	highp vec3 normal = normalize( vNormal );
	
	lowp vec3 lightDir = normalize( lightPos[0] );
	lowp float NdotL = dot( normal, lightDir );
	
	lowp float diffuseIntensity = max( NdotL, 0.0 );

#if defined( SHADOW_MAP ) || defined( SHADOW_BLOB )
	lowp float shadowIntensity = 1.0;
#endif
	
#ifdef SHADOW_MAP
	bool frontFacedToLight = ( NdotL >= 0.0 ); 
	bvec4 edgesCorrect = bvec4( vPosShadowMapSpace.x >= 0.0, vPosShadowMapSpace.x < 1.0, vPosShadowMapSpace.y >= 0.0, vPosShadowMapSpace.y < 1.0 );
	lowp float shadowValue = frontFacedToLight && all( edgesCorrect ) ? texture2DPCF( shadowMap, shadowMapSize, vPosShadowMapSpace.xy, vPosShadowMapSpace.z ) : 1.0;

		lowp float maxShadow = lightMaxShadow[0];

	shadowIntensity = mix( 1.0, shadowValue, maxShadow );
#endif
	
#ifdef SHADOW_BLOB
	shadowIntensity = min( shadowIntensity, texture2DShadowBlob( shadowBlob, vPosWS, k_shadowBlobScaleFactor ) );
#endif

#if defined( SHADOW_MAP ) || defined( SHADOW_BLOB )
		diffuseIntensity = min( diffuseIntensity, shadowIntensity );
#endif

#ifdef CHARACTER_LIGHT
	lowp float charLightDist = length( charLightPos - vPosWS );
	lowp vec3 charLightDir = normalize( vCharLightDir );
	lowp float charLightFactor = ( charLightDist - charLightMinRadius ) / ( charLightMaxRadius - charLightMinRadius );
	charLightFactor = 1.0 - clamp( charLightFactor, 0.0, 1.0 );
	
	lowp float charLightNdotL = dot( normal, charLightDir );
	lowp vec3 charLightFinalColor = charLightColor * max( charLightNdotL, 0.0 ) * charLightFactor * charLightIntensity;
#endif

#ifdef AMBIENT_MAP
	lowp vec3 lightAmb = texture2D( ambientMap, vPosAmbientMapSpace ).rgb;
#else
	lowp vec3 lightAmb = lightAmbient[0];
#endif

	diffuseIntensity *= MaxDiffuse;
	lowp float lightDiffuseFactor = mix( 1.0, diffuseIntensity, AmbientInfluence );
	
	lowp vec3 color = ( diffuseCol.rgb * mix( lightAmb, lightDiffuse[0], lightDiffuseFactor ) ) * ColorScale;
	
#ifdef CHARACTER_LIGHT
	color += diffuseCol.rgb * charLightFinalColor;
#endif

#ifdef USE_FOG
	color = mix( color * 3.0, fogColor.rgb, vFogFactor * FogInfluence );
#endif
	
	lowp vec4 finalColor = vec4( color, diffuseCol.a * AlphaScale );

	finalColor.a *= 1.0 - exp_falloff(vec3(0.0,0.0,-1.0),vPosWS,gReflectionParams.x,gReflectionParams.y);
	
	gl_FragColor = finalColor;
}
