uniform lowp float ColorScale;
uniform lowp float AlphaScale;
uniform lowp float AmbientIntensity;

#ifdef USE_FOG
uniform lowp vec4 fogColor;
#endif

uniform sampler2D diffuseMap;

varying highp vec3 vNormal;
varying highp vec3 vLightDir;
varying highp vec2 vUv;

#ifdef USE_FOG
varying lowp float vFogFactor;
#endif

void main()
{
	lowp vec4 diffuseCol = jetAlphaTexture2D( diffuseMap, vUv );
	highp vec3 normal = normalize( vNormal );
	
	lowp float NdotL = dot( normal, vLightDir );
	lowp float diffuseIntensity = max( NdotL, AmbientIntensity );
	
	lowp vec3 color = diffuseCol.rgb * diffuseIntensity * ColorScale;

#ifdef USE_FOG	
	color = mix( color, fogColor.rgb, vFogFactor );
#endif
	
	lowp vec4 finalColor = vec4( color, diffuseCol.a * AlphaScale );
	gl_FragColor = finalColor;
}
