uniform highp mat4 worldMatrix;
uniform highp mat4 worldMatrixI;
uniform highp mat4 worldViewProjMatrix;

#ifdef USE_FOG
uniform highp mat4 worldViewMatrix;
#endif

uniform lowp vec3 lightPos[MAX_LIGHTS];

#ifdef USE_FOG
uniform lowp vec4 fogColor;
uniform lowp float fogStart;
uniform lowp float fogRangeI;
#endif

#ifdef USE_SHADOWS
uniform highp mat4 shadowMapVPMatrix;
#endif

#ifdef CHARACTER_LIGHT
uniform lowp vec3 charLightPos;
#endif

attribute vec4 inPos;
attribute vec4 inNormal;
attribute vec4 inTangent;
attribute vec2 inUV0;

varying highp vec3 vLightDir;
varying highp vec2 vUv;

#ifdef USE_FOG
varying lowp float vFogFactor;
#endif

#ifdef USE_SHADOWS
varying highp vec3 vPosShadowMapSpaceBiased;
#endif

#ifdef CHARACTER_LIGHT
varying highp vec3 vPosWS;
varying highp vec3 vCharLightDir;
#endif


void main()
{	
	highp vec3 posOS = inPos.xyz;
	highp vec3 posWS = vec3( worldMatrix * vec4( posOS, 1.0 ) );
	
	highp vec3 normal = inNormal.xyz;
	highp vec3 tangent = inTangent.xyz * inTangent.w;	// for tangents w coordinate may contain handness (checkout collada2pig sources)
	highp vec3 bitangent = cross( normal, tangent );
	highp mat3 tbn = mat3( tangent, bitangent, normal );
	
	vLightDir = normalize( lightPos[0] ) * tbn;
	vUv = inUV0;
	
#ifdef USE_FOG
	lowp float vertDepth = -( worldViewMatrix * vec4( posOS, 1.0 ) ).z;
	lowp float fog = clamp( ( vertDepth - fogStart ) * fogRangeI, 0.0, 1.0 );
	vFogFactor = fog * fogColor.a;
#endif

#ifdef USE_SHADOWS
	vPosShadowMapSpaceBiased = vec3( shadowMapVPMatrix * vec4( posWS, 1.0 ) ) * 0.5 + 0.5;
#endif
	
#ifdef CHARACTER_LIGHT
	vPosWS = posWS;
	
	highp mat3 worldMatrixI3 = mat3( worldMatrixI[0].xyz, worldMatrixI[1].xyz, worldMatrixI[2].xyz );
	vCharLightDir =  transpose( tbn ) * worldMatrixI3 * ( charLightPos - posWS );
#endif

	gl_Position = worldViewProjMatrix * vec4( posOS, 1.0 );
}