uniform highp sampler2D texture0;
				
varying highp vec2 uv0;
varying   highp vec4 v_color;

uniform highp vec4 tintColor;
uniform highp float rangeAngle;
uniform highp float rangeDepth;



void main()
{
	highp float k_pi = 3.1415926;
	lowp float k_circleRadius = 0.5;
	lowp vec2 upVector = vec2(0.0, 1.0);

	highp vec4 texColor = texture2D( texture0, uv0 ) * v_color;

	highp vec2 pixelPos = uv0 - vec2(k_circleRadius, k_circleRadius);

	//-- First: Pixels should fade away as they go far from the center
	highp float targetAlpha = rangeDepth - length(pixelPos);

	//-- Second: Pixels fade away as they go very close to the center

	// This is a value starts as high as 1 and decreases to 0 quickly so we can have smooth triangle head (near the player)
	highp float triangleStartInvAlpha = 1.0-length(pixelPos)*8.0;
	triangleStartInvAlpha = clamp(triangleStartInvAlpha, 0.0, 1.0);

	targetAlpha -= triangleStartInvAlpha;

	//-- Third: Pixels, out of viewing cone, should be clipped
	highp float pixelAngle = acos(dot(normalize(pixelPos), upVector)) * 180.0 / k_pi;

	// Dividing angle by 2 since full viewing angle is excpected to be entered (maximum is 360 degrees)
	if(pixelAngle > rangeAngle * 0.5) 
	{
		targetAlpha = 0.0;
	}

	texColor.a = clamp(targetAlpha * 2.0, 0.0, 1.0);

	gl_FragColor = vec4(tintColor.rgb, texColor.a * tintColor.a);
}