
// Returns the intensity of shadow blob texture
// Note that this function has an implicit parameter 'ShadowBlobPositions'
// Make sure you have it in your shader

#ifndef SIMPLE_SHADOWBLOBS
const lowp float k_blendDistanceFactor = 0.5;
#endif

lowp float texture2DShadowBlobInner( sampler2D shadowBlobSampler, lowp vec3 fragPos, lowp vec3 shadowBlobPos, lowp float blobScaleFactor )
{
	lowp float intensity = texture2D( shadowBlob, ( fragPos.xy - shadowBlobPos.xy ) * blobScaleFactor + 0.5 ).r;

#ifndef SIMPLE_SHADOWBLOBS
	// hide shadow blobs if we too far from surface by Z coordinate (prevents drawing shadow blob on two floors)

	lowp float blendFactor = min( abs( fragPos.z - shadowBlobPos.z ) * k_blendDistanceFactor, 1.0 );
	intensity = mix( intensity, 1.0, blendFactor );
#endif

	return intensity;
}

lowp float texture2DShadowBlob( sampler2D shadowBlobSampler, lowp vec3 fragPos, lowp float blobScaleFactor )
{
	lowp float shadowBlobIntensities[16];
	shadowBlobIntensities[0] 	= texture2DShadowBlobInner( shadowBlob, fragPos, ShadowBlobPositions[0], blobScaleFactor );
	shadowBlobIntensities[1] 	= texture2DShadowBlobInner( shadowBlob, fragPos, ShadowBlobPositions[1], blobScaleFactor );
	shadowBlobIntensities[2] 	= texture2DShadowBlobInner( shadowBlob, fragPos, ShadowBlobPositions[2], blobScaleFactor );
	shadowBlobIntensities[3] 	= texture2DShadowBlobInner( shadowBlob, fragPos, ShadowBlobPositions[3], blobScaleFactor );
	shadowBlobIntensities[4] 	= texture2DShadowBlobInner( shadowBlob, fragPos, ShadowBlobPositions[4], blobScaleFactor );
	shadowBlobIntensities[5] 	= texture2DShadowBlobInner( shadowBlob, fragPos, ShadowBlobPositions[5], blobScaleFactor );
	shadowBlobIntensities[6] 	= texture2DShadowBlobInner( shadowBlob, fragPos, ShadowBlobPositions[6], blobScaleFactor );
	shadowBlobIntensities[7] 	= texture2DShadowBlobInner( shadowBlob, fragPos, ShadowBlobPositions[7], blobScaleFactor );
	shadowBlobIntensities[8] 	= texture2DShadowBlobInner( shadowBlob, fragPos, ShadowBlobPositions[8], blobScaleFactor );
	shadowBlobIntensities[9] 	= texture2DShadowBlobInner( shadowBlob, fragPos, ShadowBlobPositions[9], blobScaleFactor );
	shadowBlobIntensities[10] 	= texture2DShadowBlobInner( shadowBlob, fragPos, ShadowBlobPositions[10], blobScaleFactor );
	shadowBlobIntensities[11] 	= texture2DShadowBlobInner( shadowBlob, fragPos, ShadowBlobPositions[11], blobScaleFactor );
	shadowBlobIntensities[12] 	= texture2DShadowBlobInner( shadowBlob, fragPos, ShadowBlobPositions[12], blobScaleFactor );
	shadowBlobIntensities[13] 	= texture2DShadowBlobInner( shadowBlob, fragPos, ShadowBlobPositions[13], blobScaleFactor );
	shadowBlobIntensities[14] 	= texture2DShadowBlobInner( shadowBlob, fragPos, ShadowBlobPositions[14], blobScaleFactor );
	shadowBlobIntensities[15] 	= texture2DShadowBlobInner( shadowBlob, fragPos, ShadowBlobPositions[15], blobScaleFactor );
	
	lowp float shadowIntensity = min( shadowBlobIntensities[0], shadowBlobIntensities[1] );
	shadowIntensity = min( shadowIntensity, shadowBlobIntensities[2] );
	shadowIntensity = min( shadowIntensity, shadowBlobIntensities[3] );
	shadowIntensity = min( shadowIntensity, shadowBlobIntensities[4] );
	shadowIntensity = min( shadowIntensity, shadowBlobIntensities[5] );
	shadowIntensity = min( shadowIntensity, shadowBlobIntensities[6] );
	shadowIntensity = min( shadowIntensity, shadowBlobIntensities[7] );
	shadowIntensity = min( shadowIntensity, shadowBlobIntensities[8] );
	shadowIntensity = min( shadowIntensity, shadowBlobIntensities[9] );
	shadowIntensity = min( shadowIntensity, shadowBlobIntensities[10] );
	shadowIntensity = min( shadowIntensity, shadowBlobIntensities[11] );
	shadowIntensity = min( shadowIntensity, shadowBlobIntensities[12] );
	shadowIntensity = min( shadowIntensity, shadowBlobIntensities[13] );
	shadowIntensity = min( shadowIntensity, shadowBlobIntensities[14] );
	shadowIntensity = min( shadowIntensity, shadowBlobIntensities[15] );
	
	return shadowIntensity;
}