
// Returns the intensity of shadow blob texture
// Note that this function has an implicit parameter 'ShadowBlobPositions'
// Make sure you have it in your shader

#ifndef SIMPLE_SHADOWBLOBS
const lowp float k_blendDistanceFactor = 0.5;
#endif

lowp float texture2DShadowBlobInner( sampler2D shadowBlobSampler, lowp vec3 fragPos, lowp vec3 shadowBlobPos, lowp float blobScaleFactor )
{
	lowp float intensity = texture2D( shadowBlob, ( fragPos.xy - shadowBlobPos.xy ) * blobScaleFactor + 0.5 ).r;

#ifndef SIMPLE_SHADOWBLOBS
	// hide shadow blobs if we too far from surface by Z coordinate (prevents drawing shadow blob on two floors)

	lowp float blendFactor = min( abs( fragPos.z - shadowBlobPos.z ) * k_blendDistanceFactor, 1.0 );
	intensity = mix( intensity, 1.0, blendFactor );
#endif

	return intensity;
}

lowp float texture2DShadowBlob( sampler2D shadowBlobSampler, lowp vec3 fragPos, lowp float blobScaleFactor )
{
	return texture2DShadowBlobInner( shadowBlob, fragPos, ShadowBlobPositions[0], blobScaleFactor );
}